// refs.cc
// Latest revision: 10-23-2020
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <ctype.h>
#include <string.h>

#define uchar unsigned char
#define uint unsigned int
#define ulong unsigned long
#define ushort unsigned short int
#define LEN 20000
void clean_string(unsigned char *s, int len);
static uchar s2[LEN];


int main(int argc, char **argv)
{
   FILE *fp1;
   char filename[1024]="filename.txt";
   uchar s1[LEN];       //// input string
   memset(s1, 0, LEN);  //// input string
   memset(s2, 0, LEN);  //// output string
 
   fprintf(stderr, "refs - Converts a series of pdf files to text and creates a\
 master index consisting of filename + first %d characters\n", 10000);
   fprintf(stderr, "Requires pdf2txt, which is in python-pdfminer_20110515+dfsg-1_all.deb\n\
Note: pdf2txt does not translate greek letters correctly.\n");
   fprintf(stderr, "Usage: refs [filename] > outputfile\n\
Then use grep -c -H [searchterm] [outputfile] to find the number of matches.\n\
Only the first %d characters of the file are used.\n",LEN);

   strcpy(filename, argv[1]);
   fp1 = fopen(filename, "rb");
   fread(s1, LEN, 1, fp1);
   clean_string(s1, LEN);
   fclose(fp1);
   s2[LEN-1]=0;  // null terminate the string
   fprintf(stdout, "%s %s\n",filename, s2);
   exit(EXIT_SUCCESS);
}

void clean_string(uchar *s, int len)
{
   int k;
   int pos = 0; // string pos in s2 output string
   for(k=0; k<len; k++)
    {  
       //// get rid of multibyte chars to allow text searching
       //// increment k by number of chars in multibyte char minus 1
       //// increment pos by size of replacement string
       switch(s[k])
       {    
            case 0x0a: s2[pos] = ' ';  pos++;              //// \r = 0a
                       if (s[k+1]==0x64){ k++; }           //// bullet = 0a 64
                       break;                           
            case 0x0c: s2[pos] = ' ';  pos++;              //// \n = 0c
                       break;                           
            case 0x2d: if (s[k+1]==0x0a){ k++; pos--;}           //// hyphen + line break, drop
                       else s2[pos] = '-'; pos++;          //// ordinary hyphen
                       break;       
            case 0x2e: s2[pos] = '.'; pos++; break;
            case 0xc2: if (s[k+1]==0xaa){ memcpy(s2+pos, "(c)", 3); pos+=3; k++; }        //// copyright
                       break;
            case 0xe2: if (s[k+1]==0x80 && s[k+2]==0x93){ s2[pos]='-'; pos++; k+=2; }     //// en dash
                       if (s[k+1]==0x80 && s[k+2]==0x94){ s2[pos]='-'; pos++; k+=2; }     //// em dash
                       if (s[k+1]==0x88 && s[k+2]==0xbc){ memcpy(s2, "approx ", 7); pos+=7; k+=2; } //// approx
                       if (s[k+1]==0x88 && s[k+2]==0x92){ s2[pos]='-'; pos++; k+=2; }     //// minus
                       if (s[k+1]==0x80 && s[k+2]==0x99){ s2[pos]=0x27;pos++; k+=2; }     //// apostrophe
                       break;
            case 0xce: if (s[k+1]==0xb1){ memcpy(s2+pos, "alpha ", 6); pos+=6; k++; } //// alpha
                       if (s[k+1]==0xb2){ memcpy(s2+pos, "beta ", 5);  pos+=5; k++; } //// beta
                       if (s[k+1]==0xb3){ memcpy(s2+pos, "gamma ", 6); pos+=6; k++; } //// gamma
                       if (s[k+1]==0xb4){ memcpy(s2+pos, "delta ", 6); pos+=6; k++;  }//// delta
                       if (s[k+1]==0xb5){ memcpy(s2+pos, "epsilon ",8);pos+=8; k++; }  //// epsilon
                       if (s[k+1]==0xb6){ memcpy(s2+pos, "zeta ", 5);  pos+=5; k++; } //// zeta
                       if (s[k+1]==0xb7){ memcpy(s2+pos, "eta ", 4);   pos+=4; k++; } //// eta
                       if (s[k+1]==0xb8){ memcpy(s2+pos, "theta ", 6); pos+=6; k++; } //// theta
                       if (s[k+1]==0xb9){ memcpy(s2+pos, "iota ", 5);  pos+=5; k++; } //// iota
                       if (s[k+1]==0xba){ memcpy(s2+pos, "kappa ", 6); pos+=6; k++; } //// kappa
                       if (s[k+1]==0xbb){ memcpy(s2+pos, "lambda ", 7);pos+=7; k++; } //// lambda
                       if (s[k+1]==0xbc){ memcpy(s2+pos, "micro ", 6); pos+=6; k++; } //// mu
                       if (s[k+1]==0xbd){ memcpy(s2+pos, "nu ", 3);    pos+=3; k++; } //// nu
                       if (s[k+1]==0xbe){ memcpy(s2+pos, "xi ", 3);    pos+=3; k++; } //// xi
                       if (s[k+1]==0xa6){ memcpy(s2+pos, "phi ", 4);   pos+=4; k++; } //// phi
                       if (s[k+1]==0xb1){ memcpy(s2+pos, "+/- ", 4);   pos+=4; k++; } //// phi
                       if (s[k+1]==0x97){ s2[pos]='x';  pos++; } //// times
                       break;
            case 0xef: if (s[k+1]==0xac && s[k+2]==0x81){ s2[pos]='f'; s2[pos+1]='i'; pos+=2; k+=2; } //// fi ligature
                       if (s[k+1]==0xac && s[k+2]==0x82){ s2[pos]='f'; s2[pos+1]='l'; pos+=2; k+=2; } //// fl ligature
                       break;  
            default: s2[pos] = s[k]; pos++;
       }
//       printf("%2x ",s2[pos-1]);
    }
    s2[LEN-1]=0;  // null terminate the string
//    printf("\n>>>%s<<<\n",s2);
}

